<!DOCTYPE html>
{% load staticfiles %}
<html>
<head>
    <script type="text/javascript">
        window.onerror = function (msg) {
            document.getElementById('body').setAttribute("JSError", msg);
        }
    </script>
    {{ form.media.css }}
</head>
<body id="body">
<form method="post">
    {% csrf_token %}
    {{ form }}
    <input type="submit" name="save" value="Save"/>
    <button type="submit" name="save_continue" value="continue_value">Save &amp; continue</button>
</form>
{{ form.media.js }}
</body>